﻿/****************** 
 * Illusion2 Test *
 ******************/

import { PsychoJS } from './lib/core-2020.1.js';
import * as core from './lib/core-2020.1.js';
import { TrialHandler } from './lib/data-2020.1.js';
import { Scheduler } from './lib/util-2020.1.js';
import * as util from './lib/util-2020.1.js';
import * as visual from './lib/visual-2020.1.js';
import * as sound from './lib/sound-2020.1.js';

// init psychoJS:
const psychoJS = new PsychoJS({
  debug: true
});

// open window:
psychoJS.openWindow({
  fullscr: true,
  color: new util.Color([0, 0, 0]),
  units: 'height',
  waitBlanking: true
});

// store info about the experiment session:
let expName = 'Illusion2';  // from the Builder filename that created this script
let expInfo = {'participant': '', 'session': '001'};

// schedule the experiment:
psychoJS.schedule(psychoJS.gui.DlgFromDict({
  dictionary: expInfo,
  title: expName
}));

const flowScheduler = new Scheduler(psychoJS);
const dialogCancelScheduler = new Scheduler(psychoJS);
psychoJS.scheduleCondition(function() { return (psychoJS.gui.dialogComponent.button === 'OK'); }, flowScheduler, dialogCancelScheduler);

// flowScheduler gets run if the participants presses OK
flowScheduler.add(updateInfo); // add timeStamp
flowScheduler.add(experimentInit);
flowScheduler.add(A_StartRoutineBegin());
flowScheduler.add(A_StartRoutineEachFrame());
flowScheduler.add(A_StartRoutineEnd());
flowScheduler.add(B_ConsentRoutineBegin());
flowScheduler.add(B_ConsentRoutineEachFrame());
flowScheduler.add(B_ConsentRoutineEnd());
flowScheduler.add(C_VideoRoutineBegin());
flowScheduler.add(C_VideoRoutineEachFrame());
flowScheduler.add(C_VideoRoutineEnd());
const trialsLoopScheduler = new Scheduler(psychoJS);
flowScheduler.add(trialsLoopBegin, trialsLoopScheduler);
flowScheduler.add(trialsLoopScheduler);
flowScheduler.add(trialsLoopEnd);
flowScheduler.add(E_EndRoutineBegin());
flowScheduler.add(E_EndRoutineEachFrame());
flowScheduler.add(E_EndRoutineEnd());
flowScheduler.add(quitPsychoJS, '', true);

// quit if user presses Cancel in dialog box:
dialogCancelScheduler.add(quitPsychoJS, '', false);

psychoJS.start({
  expName: expName,
  expInfo: expInfo,
  });


var frameDur;
function updateInfo() {
  expInfo['date'] = util.MonotonicClock.getDateStr();  // add a simple timestamp
  expInfo['expName'] = expName;
  expInfo['psychopyVersion'] = '2020.1.2';
  expInfo['OS'] = window.navigator.platform;

  // store frame rate of monitor if we can measure it successfully
  expInfo['frameRate'] = psychoJS.window.getActualFrameRate();
  if (typeof expInfo['frameRate'] !== 'undefined')
    frameDur = 1.0 / Math.round(expInfo['frameRate']);
  else
    frameDur = 1.0 / 60.0; // couldn't get a reliable measure so guess

  // add info from the URL:
  util.addInfoFromUrl(expInfo);
  
  return Scheduler.Event.NEXT;
}


var A_StartClock;
var instText;
var key_resp;
var VideoPath;
var B_ConsentClock;
var ConsentForm1;
var Button1;
var Mouse1;
var ConsentForm22;
var Button22;
var Mouse22;
var ConsentFrom33;
var Button_Agreee;
var Button_Disagreee;
var Mouse33;
var C_VideoClock;
var movieClock;
var movie;
var D_ResponseClock;
var TextFrame;
var instr;
var text;
var E_EndClock;
var text_2;
var globalClock;
var routineTimer;
function experimentInit() {
  // Initialize components for Routine "A_Start"
  A_StartClock = new util.Clock();
  instText = new visual.TextStim({
    win: psychoJS.window,
    name: 'instText',
    text: 'In this demo, we show how you can use text components to write a users keyboard input on screen.\n\nYou will be presented with a target word. Then, in the next routine, you will be asked to type that word.\nWhen you have finished typing the word, press "return" to end the trial.\n\n\nPress spacebar to continue...',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: 0.0 
  });
  
  key_resp = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  VideoPath = "Videos/FrameTEST2250.mp4";
  
  // Initialize components for Routine "B_Consent"
  B_ConsentClock = new util.Clock();
  ConsentForm1 = new visual.ImageStim({
    win : psychoJS.window,
    name : 'ConsentForm1', units : 'height', 
    image : 'Images/consent1.jpg', mask : undefined,
    ori : 0, pos : [0, 0], size : [1.4, 1.0],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : 0.0 
  });
  Button1 = new visual.ImageStim({
    win : psychoJS.window,
    name : 'Button1', units : 'height', 
    image : 'Images/button.png', mask : undefined,
    ori : 0, pos : [0.65, (- 0.45)], size : [0.1, 0.045],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -1.0 
  });
  Mouse1 = new core.Mouse({
    win: psychoJS.window,
  });
  Mouse1.mouseClock = new util.Clock();
  ConsentForm22 = new visual.ImageStim({
    win : psychoJS.window,
    name : 'ConsentForm22', units : 'height', 
    image : 'Images/consent2.jpg', mask : undefined,
    ori : 0, pos : [0, 0], size : [1.4, 0.7],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -3.0 
  });
  Button22 = new visual.ImageStim({
    win : psychoJS.window,
    name : 'Button22', units : 'height', 
    image : 'Images/button.png', mask : undefined,
    ori : 0, pos : [0.65, (- 0.45)], size : [0.1, 0.045],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -4.0 
  });
  Mouse22 = new core.Mouse({
    win: psychoJS.window,
  });
  Mouse22.mouseClock = new util.Clock();
  ConsentFrom33 = new visual.ImageStim({
    win : psychoJS.window,
    name : 'ConsentFrom33', units : 'height', 
    image : 'Images/consent3.jpg', mask : undefined,
    ori : 0, pos : [0, 0], size : [1.4, 0.6],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -6.0 
  });
  Button_Agreee = new visual.ImageStim({
    win : psychoJS.window,
    name : 'Button_Agreee', units : 'height', 
    image : 'Images/agree.png', mask : undefined,
    ori : 0, pos : [(- 0.7), (- 0.45)], size : [0.15, 0.045],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -7.0 
  });
  Button_Disagreee = new visual.ImageStim({
    win : psychoJS.window,
    name : 'Button_Disagreee', units : 'height', 
    image : 'Images/disagree.png', mask : undefined,
    ori : 0, pos : [0.7, (- 0.45)], size : [0.15, 0.045],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -8.0 
  });
  Mouse33 = new core.Mouse({
    win: psychoJS.window,
  });
  Mouse33.mouseClock = new util.Clock();
  // Initialize components for Routine "C_Video"
  C_VideoClock = new util.Clock();
  movieClock = new util.Clock();
  movie = new visual.MovieStim({
    win: psychoJS.window,
    name: 'movie',
    units: 'height',
    movie: VideoPath,
    pos: [0, 0],
    size: [1.6, 1],
    ori: 0,
    opacity: 1,
    loop: false,
    noAudio: true,
    });
  // Initialize components for Routine "D_Response"
  D_ResponseClock = new util.Clock();
  TextFrame = new visual.Rect ({
    win: psychoJS.window, name: 'TextFrame', units : 'height', 
    width: [0.5, 0.15][0], height: [0.5, 0.15][1],
    ori: 0, pos: [0, 0],
    lineWidth: 15, lineColor: new util.Color([(- 1), (- 1), (- 1)]),
    fillColor: new util.Color([0, 0, 0]),
    opacity: 1, depth: 0, interpolate: true,
  });
  
  instr = new visual.TextStim({
    win: psychoJS.window,
    name: 'instr',
    text: 'Type the answer in the frame below, \nand press return',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0.5], height: 0.1,  wrapWidth: undefined, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: -1.0 
  });
  
  text = new visual.TextStim({
    win: psychoJS.window,
    name: 'text',
    text: '',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.1,  wrapWidth: undefined, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: -2.0 
  });
  
  // Initialize components for Routine "E_End"
  E_EndClock = new util.Clock();
  text_2 = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_2',
    text: 'Thank you for participanting in the experiment!',
    font: 'Arial',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: 0.0 
  });
  
  // Create some handy timers
  globalClock = new util.Clock();  // to track the time since experiment started
  routineTimer = new util.CountdownTimer();  // to track time remaining of each (non-slip) routine
  
  return Scheduler.Event.NEXT;
}


var t;
var frameN;
var _key_resp_allKeys;
var A_StartComponents;
function A_StartRoutineBegin(trials) {
  return function () {
    //------Prepare to start Routine 'A_Start'-------
    t = 0;
    A_StartClock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    key_resp.keys = undefined;
    key_resp.rt = undefined;
    _key_resp_allKeys = [];
    // keep track of which components have finished
    A_StartComponents = [];
    A_StartComponents.push(instText);
    A_StartComponents.push(key_resp);
    
    for (const thisComponent of A_StartComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}


var continueRoutine;
function A_StartRoutineEachFrame(trials) {
  return function () {
    //------Loop for each frame of Routine 'A_Start'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = A_StartClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *instText* updates
    if (t >= 0.0 && instText.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      instText.tStart = t;  // (not accounting for frame time here)
      instText.frameNStart = frameN;  // exact frame index
      
      instText.setAutoDraw(true);
    }

    
    // *key_resp* updates
    if (t >= 0.0 && key_resp.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_resp.tStart = t;  // (not accounting for frame time here)
      key_resp.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_resp.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_resp.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_resp.clearEvents(); });
    }

    if (key_resp.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_resp.getKeys({keyList: ['space'], waitRelease: false});
      _key_resp_allKeys = _key_resp_allKeys.concat(theseKeys);
      if (_key_resp_allKeys.length > 0) {
        key_resp.keys = _key_resp_allKeys[_key_resp_allKeys.length - 1].name;  // just the last key pressed
        key_resp.rt = _key_resp_allKeys[_key_resp_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of A_StartComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function A_StartRoutineEnd(trials) {
  return function () {
    //------Ending Routine 'A_Start'-------
    for (const thisComponent of A_StartComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    psychoJS.experiment.addData('key_resp.keys', key_resp.keys);
    if (typeof key_resp.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('key_resp.rt', key_resp.rt);
        routineTimer.reset();
        }
    
    key_resp.stop();
    // the Routine "A_Start" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var gotValidClick;
var B_ConsentComponents;
function B_ConsentRoutineBegin(trials) {
  return function () {
    //------Prepare to start Routine 'B_Consent'-------
    t = 0;
    B_ConsentClock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    // setup some python lists for storing info about the Mouse1
    // current position of the mouse:
    Mouse1.x = [];
    Mouse1.y = [];
    Mouse1.leftButton = [];
    Mouse1.midButton = [];
    Mouse1.rightButton = [];
    Mouse1.time = [];
    Mouse1.clicked_name = [];
    gotValidClick = false; // until a click is received
    // setup some python lists for storing info about the Mouse22
    // current position of the mouse:
    Mouse22.x = [];
    Mouse22.y = [];
    Mouse22.leftButton = [];
    Mouse22.midButton = [];
    Mouse22.rightButton = [];
    Mouse22.time = [];
    Mouse22.clicked_name = [];
    gotValidClick = false; // until a click is received
    // setup some python lists for storing info about the Mouse33
    Mouse33.clicked_name = [];
    gotValidClick = false; // until a click is received
    // keep track of which components have finished
    B_ConsentComponents = [];
    B_ConsentComponents.push(ConsentForm1);
    B_ConsentComponents.push(Button1);
    B_ConsentComponents.push(Mouse1);
    B_ConsentComponents.push(ConsentForm22);
    B_ConsentComponents.push(Button22);
    B_ConsentComponents.push(Mouse22);
    B_ConsentComponents.push(ConsentFrom33);
    B_ConsentComponents.push(Button_Agreee);
    B_ConsentComponents.push(Button_Disagreee);
    B_ConsentComponents.push(Mouse33);
    
    for (const thisComponent of B_ConsentComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}


var prevButtonState;
function B_ConsentRoutineEachFrame(trials) {
  return function () {
    //------Loop for each frame of Routine 'B_Consent'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = B_ConsentClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *ConsentForm1* updates
    if (frameN >= 0.0 && ConsentForm1.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ConsentForm1.tStart = t;  // (not accounting for frame time here)
      ConsentForm1.frameNStart = frameN;  // exact frame index
      
      ConsentForm1.setAutoDraw(true);
    }

    if (ConsentForm1.status === PsychoJS.Status.STARTED && Boolean((Mouse1.clicked_name == ['Button1']))) {
      ConsentForm1.setAutoDraw(false);
    }
    
    // *Button1* updates
    if (frameN >= 0.0 && Button1.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Button1.tStart = t;  // (not accounting for frame time here)
      Button1.frameNStart = frameN;  // exact frame index
      
      Button1.setAutoDraw(true);
    }

    if (Button1.status === PsychoJS.Status.STARTED && Boolean((Mouse1.clicked_name == ['Button1']))) {
      Button1.setAutoDraw(false);
    }
    // *Mouse1* updates
    if (frameN >= 0.0 && Mouse1.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Mouse1.tStart = t;  // (not accounting for frame time here)
      Mouse1.frameNStart = frameN;  // exact frame index
      
      Mouse1.status = PsychoJS.Status.STARTED;
      Mouse1.mouseClock.reset();
      prevButtonState = Mouse1.getPressed();  // if button is down already this ISN'T a new click
      }
    if (Mouse1.status === PsychoJS.Status.STARTED && Boolean((Mouse1.clicked_name == ['Button1']))) {
      Mouse1.status = PsychoJS.Status.FINISHED;
  }
    if (Mouse1.status === PsychoJS.Status.STARTED) {  // only update if started and not finished!
      let buttons = Mouse1.getPressed();
      if (!buttons.every( (e,i,) => (e == prevButtonState[i]) )) { // button state changed?
        prevButtonState = buttons;
        if (buttons.reduce( (e, acc) => (e+acc) ) > 0) { // state changed to a new click
          const xys = Mouse1.getPos();
          Mouse1.x.push(xys[0]);
          Mouse1.y.push(xys[1]);
          Mouse1.leftButton.push(buttons[0]);
          Mouse1.midButton.push(buttons[1]);
          Mouse1.rightButton.push(buttons[2]);
          Mouse1.time.push(Mouse1.mouseClock.getTime());
          // check if the mouse was inside our 'clickable' objects
          gotValidClick = false;
          for (const obj of [Button1]) {
            if (obj.contains(Mouse1)) {
              gotValidClick = true;
              Mouse1.clicked_name.push(obj.name)
            }
          }
        }
      }
    }
    
    // *ConsentForm22* updates
    if (((Mouse1.clicked_name == ['Button1'])) && ConsentForm22.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ConsentForm22.tStart = t;  // (not accounting for frame time here)
      ConsentForm22.frameNStart = frameN;  // exact frame index
      
      ConsentForm22.setAutoDraw(true);
    }

    if (ConsentForm22.status === PsychoJS.Status.STARTED && Boolean((Mouse22.clicked_name == ['Button22']))) {
      ConsentForm22.setAutoDraw(false);
    }
    
    // *Button22* updates
    if (((Mouse1.clicked_name == ['Button1'])) && Button22.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Button22.tStart = t;  // (not accounting for frame time here)
      Button22.frameNStart = frameN;  // exact frame index
      
      Button22.setAutoDraw(true);
    }

    if (Button22.status === PsychoJS.Status.STARTED && Boolean((Mouse22.clicked_name == ['Button22']))) {
      Button22.setAutoDraw(false);
    }
    // *Mouse22* updates
    if (((Mouse1.clicked_name == ['Button1'])) && Mouse22.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Mouse22.tStart = t;  // (not accounting for frame time here)
      Mouse22.frameNStart = frameN;  // exact frame index
      
      Mouse22.status = PsychoJS.Status.STARTED;
      Mouse22.mouseClock.reset();
      prevButtonState = Mouse22.getPressed();  // if button is down already this ISN'T a new click
      }
    if (Mouse22.status === PsychoJS.Status.STARTED && Boolean((Mouse22.clicked_name == ['Button22']))) {
      Mouse22.status = PsychoJS.Status.FINISHED;
  }
    if (Mouse22.status === PsychoJS.Status.STARTED) {  // only update if started and not finished!
      let buttons = Mouse22.getPressed();
      if (!buttons.every( (e,i,) => (e == prevButtonState[i]) )) { // button state changed?
        prevButtonState = buttons;
        if (buttons.reduce( (e, acc) => (e+acc) ) > 0) { // state changed to a new click
          const xys = Mouse22.getPos();
          Mouse22.x.push(xys[0]);
          Mouse22.y.push(xys[1]);
          Mouse22.leftButton.push(buttons[0]);
          Mouse22.midButton.push(buttons[1]);
          Mouse22.rightButton.push(buttons[2]);
          Mouse22.time.push(Mouse22.mouseClock.getTime());
          // check if the mouse was inside our 'clickable' objects
          gotValidClick = false;
          for (const obj of [Button22]) {
            if (obj.contains(Mouse22)) {
              gotValidClick = true;
              Mouse22.clicked_name.push(obj.name)
            }
          }
        }
      }
    }
    
    // *ConsentFrom33* updates
    if (((Mouse22.clicked_name == ['Button22'])) && ConsentFrom33.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ConsentFrom33.tStart = t;  // (not accounting for frame time here)
      ConsentFrom33.frameNStart = frameN;  // exact frame index
      
      ConsentFrom33.setAutoDraw(true);
    }

    
    // *Button_Agreee* updates
    if (((Mouse22.clicked_name == ['Button22'])) && Button_Agreee.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Button_Agreee.tStart = t;  // (not accounting for frame time here)
      Button_Agreee.frameNStart = frameN;  // exact frame index
      
      Button_Agreee.setAutoDraw(true);
    }

    
    // *Button_Disagreee* updates
    if (((Mouse22.clicked_name == ['Button22'])) && Button_Disagreee.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Button_Disagreee.tStart = t;  // (not accounting for frame time here)
      Button_Disagreee.frameNStart = frameN;  // exact frame index
      
      Button_Disagreee.setAutoDraw(true);
    }

    // *Mouse33* updates
    if (frameN >= 0.0 && Mouse33.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      Mouse33.tStart = t;  // (not accounting for frame time here)
      Mouse33.frameNStart = frameN;  // exact frame index
      
      Mouse33.status = PsychoJS.Status.STARTED;
      Mouse33.mouseClock.reset();
      prevButtonState = Mouse33.getPressed();  // if button is down already this ISN'T a new click
      }
    if (Mouse33.status === PsychoJS.Status.STARTED) {  // only update if started and not finished!
      let buttons = Mouse33.getPressed();
      if (!buttons.every( (e,i,) => (e == prevButtonState[i]) )) { // button state changed?
        prevButtonState = buttons;
        if (buttons.reduce( (e, acc) => (e+acc) ) > 0) { // state changed to a new click
          // check if the mouse was inside our 'clickable' objects
          gotValidClick = false;
          for (const obj of [Button_Agreee]) {
            if (obj.contains(Mouse33)) {
              gotValidClick = true;
              Mouse33.clicked_name.push(obj.name)
            }
          }
          if (gotValidClick === true) { // abort routine on response
            continueRoutine = false;
          }
        }
      }
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of B_ConsentComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function B_ConsentRoutineEnd(trials) {
  return function () {
    //------Ending Routine 'B_Consent'-------
    for (const thisComponent of B_ConsentComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // store data for thisExp (ExperimentHandler)
    psychoJS.experiment.addData('Mouse1.x', Mouse1.x);
    psychoJS.experiment.addData('Mouse1.y', Mouse1.y);
    psychoJS.experiment.addData('Mouse1.leftButton', Mouse1.leftButton);
    psychoJS.experiment.addData('Mouse1.midButton', Mouse1.midButton);
    psychoJS.experiment.addData('Mouse1.rightButton', Mouse1.rightButton);
    psychoJS.experiment.addData('Mouse1.time', Mouse1.time);
    psychoJS.experiment.addData('Mouse1.clicked_name', Mouse1.clicked_name);
    
    // store data for thisExp (ExperimentHandler)
    psychoJS.experiment.addData('Mouse22.x', Mouse22.x);
    psychoJS.experiment.addData('Mouse22.y', Mouse22.y);
    psychoJS.experiment.addData('Mouse22.leftButton', Mouse22.leftButton);
    psychoJS.experiment.addData('Mouse22.midButton', Mouse22.midButton);
    psychoJS.experiment.addData('Mouse22.rightButton', Mouse22.rightButton);
    psychoJS.experiment.addData('Mouse22.time', Mouse22.time);
    psychoJS.experiment.addData('Mouse22.clicked_name', Mouse22.clicked_name);
    
    // store data for thisExp (ExperimentHandler)
    const xys = Mouse33.getPos();
    const buttons = Mouse33.getPressed();
    psychoJS.experiment.addData('Mouse33.x', xys[0]);
    psychoJS.experiment.addData('Mouse33.y', xys[1]);
    psychoJS.experiment.addData('Mouse33.leftButton', buttons[0]);
    psychoJS.experiment.addData('Mouse33.midButton', buttons[1]);
    psychoJS.experiment.addData('Mouse33.rightButton', buttons[2]);
    if (Mouse33.clicked_name.length > 0) {
      psychoJS.experiment.addData('Mouse33.clicked_name', Mouse33.clicked_name[0]);}
    // the Routine "B_Consent" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var C_VideoComponents;
function C_VideoRoutineBegin(trials) {
  return function () {
    //------Prepare to start Routine 'C_Video'-------
    t = 0;
    C_VideoClock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    // keep track of which components have finished
    C_VideoComponents = [];
    C_VideoComponents.push(movie);
    
    for (const thisComponent of C_VideoComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}


function C_VideoRoutineEachFrame(trials) {
  return function () {
    //------Loop for each frame of Routine 'C_Video'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = C_VideoClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *movie* updates
    if (t >= 0.0 && movie.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      movie.tStart = t;  // (not accounting for frame time here)
      movie.frameNStart = frameN;  // exact frame index
      
      movie.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of C_VideoComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function C_VideoRoutineEnd(trials) {
  return function () {
    //------Ending Routine 'C_Video'-------
    for (const thisComponent of C_VideoComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "C_Video" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var trials;
var currentLoop;
function trialsLoopBegin(thisScheduler) {
  // set up handler to look after randomisation of conditions etc
  trials = new TrialHandler({
    psychoJS: psychoJS,
    nReps: 1, method: TrialHandler.Method.SEQUENTIAL,
    extraInfo: expInfo, originPath: undefined,
    trialList: undefined,
    seed: undefined, name: 'trials'
  });
  psychoJS.experiment.addLoop(trials); // add the loop to the experiment
  currentLoop = trials;  // we're now the current loop

  // Schedule all the trials in the trialList:
  for (const thisTrial of trials) {
    const snapshot = trials.getSnapshot();
    thisScheduler.add(importConditions(snapshot));
    thisScheduler.add(D_ResponseRoutineBegin(snapshot));
    thisScheduler.add(D_ResponseRoutineEachFrame(snapshot));
    thisScheduler.add(D_ResponseRoutineEnd(snapshot));
    thisScheduler.add(endLoopIteration(thisScheduler, snapshot));
  }

  return Scheduler.Event.NEXT;
}


function trialsLoopEnd() {
  psychoJS.experiment.removeLoop(trials);

  return Scheduler.Event.NEXT;
}


var modify;
var D_ResponseComponents;
function D_ResponseRoutineBegin(trials) {
  return function () {
    //------Prepare to start Routine 'D_Response'-------
    t = 0;
    D_ResponseClock.reset(); // clock
    frameN = -1;
    // update component parameters for each repeat
    modify = false;
    text.text = '';
    // keep track of which components have finished
    D_ResponseComponents = [];
    D_ResponseComponents.push(TextFrame);
    D_ResponseComponents.push(instr);
    D_ResponseComponents.push(text);
    
    for (const thisComponent of D_ResponseComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}


var textAdd;
function D_ResponseRoutineEachFrame(trials) {
  return function () {
    //------Loop for each frame of Routine 'D_Response'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = D_ResponseClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *TextFrame* updates
    if (t >= 0.0 && TextFrame.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      TextFrame.tStart = t;  // (not accounting for frame time here)
      TextFrame.frameNStart = frameN;  // exact frame index
      
      TextFrame.setAutoDraw(true);
    }

    
    // *instr* updates
    if (t >= 0.0 && instr.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      instr.tStart = t;  // (not accounting for frame time here)
      instr.frameNStart = frameN;  // exact frame index
      
      instr.setAutoDraw(true);
    }

    
    // *text* updates
    if (t >= 0.0 && text.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text.tStart = t;  // (not accounting for frame time here)
      text.frameNStart = frameN;  // exact frame index
      
      text.setAutoDraw(true);
    }

    let theseKeys = psychoJS.eventManager.getKeys();
    if (theseKeys.length > 0) {  // at least one key was pressed
      textAdd = theseKeys[theseKeys.length-1]; 
      }
    
    
    if (textAdd === 'return') {
        textAdd = '';  // Add nothing
        continueRoutine = false;  // End routine (if that is what you want)
    } else if (textAdd === 'space') {
        textAdd = ' ';  // Add a space
    } else if (textAdd === 'backspace') {
        text.text = text.text.slice(0, -1);
        textAdd = undefined;
    } else if (['lshift', 'rshift'].includes(textAdd)) {
        modify = true;
    } else if (textAdd !== undefined) {
        if (modify) {
            text.text = text.text + textAdd.toUpperCase();
            modify = false;
        } else {
            text.text = text.text + textAdd
        }
        textAdd = undefined;
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of D_ResponseComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function D_ResponseRoutineEnd(trials) {
  return function () {
    //------Ending Routine 'D_Response'-------
    for (const thisComponent of D_ResponseComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    psychoJS.experiment.addData("typedWord", text.text)
    text.setText('')  // empty the onscreen text ready for next input
    // the Routine "D_Response" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var E_EndComponents;
function E_EndRoutineBegin(trials) {
  return function () {
    //------Prepare to start Routine 'E_End'-------
    t = 0;
    E_EndClock.reset(); // clock
    frameN = -1;
    routineTimer.add(5.000000);
    // update component parameters for each repeat
    // keep track of which components have finished
    E_EndComponents = [];
    E_EndComponents.push(text_2);
    
    for (const thisComponent of E_EndComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    
    return Scheduler.Event.NEXT;
  };
}


var frameRemains;
function E_EndRoutineEachFrame(trials) {
  return function () {
    //------Loop for each frame of Routine 'E_End'-------
    let continueRoutine = true; // until we're told otherwise
    // get current time
    t = E_EndClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *text_2* updates
    if (t >= 0.0 && text_2.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_2.tStart = t;  // (not accounting for frame time here)
      text_2.frameNStart = frameN;  // exact frame index
      
      text_2.setAutoDraw(true);
    }

    frameRemains = 0.0 + 5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (text_2.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      text_2.setAutoDraw(false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of E_EndComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function E_EndRoutineEnd(trials) {
  return function () {
    //------Ending Routine 'E_End'-------
    for (const thisComponent of E_EndComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    return Scheduler.Event.NEXT;
  };
}


function endLoopIteration(thisScheduler, loop) {
  // ------Prepare for next entry------
  return function () {
    if (typeof loop !== 'undefined') {
      // ------Check if user ended loop early------
      if (loop.finished) {
        // Check for and save orphaned data
        if (psychoJS.experiment.isEntryEmpty()) {
          psychoJS.experiment.nextEntry(loop);
        }
      thisScheduler.stop();
      } else {
        const thisTrial = loop.getCurrentTrial();
        if (typeof thisTrial === 'undefined' || !('isTrials' in thisTrial) || thisTrial.isTrials) {
          psychoJS.experiment.nextEntry(loop);
        }
      }
    return Scheduler.Event.NEXT;
    }
  };
}


function importConditions(trials) {
  return function () {
    psychoJS.importAttributes(trials.getCurrentTrial());
    return Scheduler.Event.NEXT;
    };
}


function quitPsychoJS(message, isCompleted) {
  // Check for and save orphaned data
  if (psychoJS.experiment.isEntryEmpty()) {
    psychoJS.experiment.nextEntry();
  }
  
  
  
  
  psychoJS.window.close();
  psychoJS.quit({message: message, isCompleted: isCompleted});
  
  return Scheduler.Event.QUIT;
}
